# /tcl/km-branch.tcl
#
# These procs have to do with the branches to questions. Branches are 
# answer-dependent questions that follow other questions.
#
# $Id: km-branch-procs.tcl,v 1.2 2009/02/13 20:40:39 jeffd Exp $

ad_proc km_get_branches { 
    {
	-question_attributes_p 0 
	-branch_id 0
    } 
    root_question_id 
    object_type_id 
} {
    Returns a list of the question_ids directly beneath this question_id in the branch tree. If
    this question is not a branched question then it returns nothing.
} {
    
    set join_tables [list "sn_question_object_type_map m1" "sn_question_object_type_map m2"]
    set join_clause ""
    set columns ""
    set limitation ""

    if { $question_attributes_p } {
	
	# We need to know the abstract_data_type of the parent question to know
        # if the answer is stored as an option or as a foreign key in the branch recipe.

	set abstract_data_type [thd [km_get_question $root_question_id $object_type_id]]
	
	switch $abstract_data_type {
	    "integer" -
	    "text" {
		append columns ", m1.branch_answer branch_answer"
	    } 
	    "category" - "other_category" {
		append columns ",c.category_id, c.long_name branch_answer"
		lappend join_tables "categories c"
		append join_clause " and m1.branch_answer_foreign_key = c.category_id(+)"
	    }  
	    "option" {
		append columns ", ao.option_id,  ao.answer_option branch_answer"
		lappend join_tables "sn_answer_options ao"
		append join_clause " and m1.branch_answer_foreign_key = ao.option_id(+)"
	    }
	    default { return }
	}
	lappend join_tables " sn_questions q"
	append join_clause " and q.question_id = m1.question_id"
	append columns ", m1.branch_operator, pretty_name, q.entry_explanation, q.presentation_type, q.abstract_data_type"
    }
    if { $branch_id } { set limitation " and m1.question_id = :branch_id " }

    set sql "select m1.question_id $columns from [join $join_tables ","] 
             where m2.question_id = m1.parent_question_id
             and m1.branch_p = 't'    
             and m1.object_type_id = :object_type_id
             and m1.parent_question_id = :root_question_id
             $join_clause $limitation"
    set result [km_db_to_named_list_list km_get_branches $sql]
    return $result
} 

ad_proc km_default_branch { root_branch_id } {
    Returns the default branch of a branch node.
} {
    set default_branch [db_string km_default_branch "
	select default_branch 
	from sn_question_object_type_map qm
        where question_id=$root_branch_id 
	    and (select qm2.branch_p 
		 from sn_question_object_type_map qm2 
		 where qm2.question_id = qm.default_branch) = 't'" -default ""]

    return $default_branch
}

ad_proc km_branch_parent { question_id object_type_id } {
    Returns the parent_question_id of this question iff this is a branch.
} {
    set result [db_string km_branch_parent "
	select parent_question_id 
	from sn_question_object_type_map 
	where question_id=:question_id 
	    and branch_p='t' 
	    and object_type_id=:object_type_id" -default ""]
    
    return $result
}

ad_proc km_composite_parent { question_id object_type_id } {
    Returns the parent_question_id of this question if this is a composite child.
} {
    set result [db_string km_composite_parent "
	select parent_question_id 
	from sn_question_object_type_map 
	where question_id=:question_id 
	    and branch_p='f'
	    and object_type_id=:object_type_id" -default ""]
    if {$result == [km_static km_dummy_object_id]} {
        return ""
    }
    return $result
}

ad_proc km_get_root_branch { question_id object_type_id} {
    Returns the id of the root question of this branch.
} { 
    set result [db_string km_get_root_branch "
    select question_id
    from sn_question_object_type_map
    where question_id <> :question_id and branch_p='f'
    start with question_id=:question_id and object_type_id=:object_type_id
    connect by question_id = prior parent_question_id" -default ""]

    return $result
}

ad_proc km_root_branch_p { question_id } {
    Returns true if this question is a root of a  branch (at any level)
} {
    return [db_string km_root_branch_p {
	select decode(count(*),0,0,1) from dual where exists
	(select 1
	 from sn_question_object_type_map 
	 where parent_question_id = :question_id and branch_p='t')
    }]
}

ad_proc km_composite_child_p { question_id object_type_id } {
    Returns 1 if this question is the child of a composite question, 0 otherwise.
    Included here because the distinction between branches and composite children is slight.
} {
    set result [db_string km_composite_child_p "
	select decode(parent_question_id,[km_static km_dummy_object_id],0,1)
	from sn_question_object_type_map
        where branch_p='f' 
	    and question_id=:question_id
	    and object_type_id=:object_type_id" -default "0"]

    return 0
}

ad_proc km_active_branch_path { root_question_id object_id object_type_id } {

    Returns the sequence of questions after this root branch question that
    are part of the active branch path for this object.

} {
    set branch_id $root_question_id
    set branch_list {}
    if ![km_root_branch_p $root_question_id] { return $branch_list }
    
    # Loop through hierarchy until we reach a leaf node or an unanswered question.
    set i 0
    while { 1 == 1 } {
	# Get object data for the branch_id.
	set question [km_get_question $branch_id $object_type_id]
	if [empty_string_p $question] {
	    ns_log Error "Question $branch_id not found in km_active_branch_path"
	    return
	}

	set answer [fst [km_get_object_data -answers_only_p 0 -questions [list {question_id pretty_name abstract_data_type} $question] $object_id]]
	set value [fst $answer]
	set abstract_data_type [lindex $answer 3]

	# Append this question to the list
	lappend branch_list $branch_id

	if ![km_answered_p $abstract_data_type $value] {
	    # the answer is empty so return the list
	    return $branch_list 
	} else {
	    # Get next branch
	    set branch_id [km_next_branch -abstract_data_type $abstract_data_type $branch_id $value $object_type_id]
	    if [empty_string_p $branch_id ] { return $branch_list }
	}

	incr i
	if { $i > 100 } {
	    ns_log Error "Endless loop in km_active_branch_path"
	    return
	}
    }
}

ad_proc km_active_path_p { question_id object_id object_type_id } {

    Returns 1 if this question is on an active branch path or is not on a branch at all. 
    Returns 0 if this question is not on a central branch path (i.e. it is decoupled 
    from the normal question sequence)

} {
    set root_question_id [km_get_root_branch $question_id $object_type_id]
    if [empty_string_p $root_question_id] { return 1 }

    set active_branches [km_active_branch_path $root_question_id $object_id $object_type_id]
    if [elem_p $question_id $active_branches] {
	return 1
    } else {
	return 0
    }
}

ad_proc km_next_branch { { -abstract_data_type "" } root_branch_id answer object_type_id } {
    
    Returns the question_id that should follow this question as a result of
    the answers. It returns the default question if the answer hasn't been matched or is empty.
    If it can't figure out the next question it returns nothing.

} {
    if [empty_string_p $abstract_data_type] {
	set abstract_data_type [km_static question_abstract_data_type $root_branch_id]
    }

    # If it's not one of these abstract data types then this proc should never have been called!
    if { [lsearch {"category" "other_category" "option"} $abstract_data_type] >= 0 } {

	# Data answers are always tuples - we need the first value of the first pair
	set answer [fst [fst $answer]]
	
	# Then the answer is the integer value of the option, a foreign key
	set column "branch_answer_foreign_key"

	if { [empty_string_p $answer] || ![integer_p $answer] } { return }

    } elseif { [lsearch {"text" "integer"} $abstract_data_type] >= 0 } {
	set column "branch_answer"
	set answer [fst $answer]
	if { [empty_string_p $answer] || ![integer_p $answer] } { return }
    } else { 
	return ""
    }

    set default_branch [km_default_branch $root_branch_id]

    set branches [db_list_of_lists km_next_branch "
	select question_id, branch_operator, $column 
	from sn_question_object_type_map
        where parent_question_id=$root_branch_id 
	    and object_type_id=$object_type_id"]

    # Go through each of the branches and see if the operator matches this value
    foreach branch $branches {
	set branch_id [fst $branch]
	set operator [snd $branch]
	set branch_answer [thd $branch]

	if [eval_binary $operator $answer $branch_answer] { return $branch_id }
    }

    return $default_branch
}

proc km_pretty_operator { operator } {
    switch $operator {
	"=" { return "Equals =" }
	"<" { return "Less Than <" }
	">" { return "Greater Than >" }
	default { return }
    }
}

ad_proc km_branch_questions {
    {
	-branch_id 0
	-all_properties_p 1
	-root_question_id 0
	-composite_children_p 1
    }
    object_type_id
} {

    This proc returns keyed question lists with attributes on the basis of variable branch
    properties. i.e. It can return a list that shows all questions of an object_type that
    are NOT branches, or these plus the one that IS this branch_id.

    This proc has become more a general admin questions list, since it can do some stuff 
    for composite questions, too.

} {
    if { $all_properties_p } {
	set additional_columns ",presentation_type, tag_height, tag_width, entry_explanation, qm.mandatory_p, qm.question_state, qm.form_number, qm.sort_key "
    }

    set limitation "parent_question_id=[km_static km_dummy_object_id] and question_state <> 'invisible'"
    set counter 1

    if { $root_question_id } { 
	append limitation " and qm.question_id != :root_question_id" 
	incr counter
    }
    if { $branch_id } {
	append limitation " or qm.question_id = :branch_id " 
	incr counter
    } 
    if { $counter > 1 } { 
	set limitation " and ($limitation)"
    } else {
	set limitation " and $limitation"
    }
    if { $composite_children_p } {
	append limitation "
and q.abstract_data_type in ('text', 'option', 'category', 'other_category', 'date', 'file', 'integer', 'object_link', 'content_link', 'user_link', 'child_object', 'nephew_object')
and q.question_id not in (select parent_question_id from sn_question_object_type_map where object_type_id=:object_type_id and branch_p='t')"
    }
    set result [km_db_to_named_list_list km_branch_questions "
	select q.question_id, q.pretty_name, q.abstract_data_type
	$additional_columns 
        from sn_questions q, sn_question_object_type_map qm
        where q.question_id=qm.question_id
	    and qm.object_type_id = :object_type_id 
	    $limitation
       	order by qm.sort_key"]

    return $result
}
