# /tcl/km-callback-procs.tcl
#
# Procs called from other modules.
#
# @author Carsten Clasohm (carsten@arsdigita.de)
# @creation-date 2001-10-10
# @cvs-id $Id: km-callback-procs.tcl,v 1.1 2003/07/02 12:19:41 peterm Exp $

ad_proc -public library_post_instantiation { 
    package_id
} {
    This is the TCL proc that is called automatically by the APM
    whenever a new instance of the application is created.
} {
    sn_community::grant_role_permissions $package_id {
	maintainer {access create km_publish}
	member {access create}
	guest {access}
    }
    db_exec_plsql create_magic_psn_category_objects {
	begin
	  psn_res_application.create_magic_objects (
	    package_id => :package_id
          );
	end;
    }
}

ad_proc -public library_supported_privileges {} {

    Returns a list of instance-level privileges supported by the Library.
    Used by sn-communities to set permissions for role groups.

} {
    return {
	{admin "Administer"} 
	{access "Access"} 
	{create "Create new objects"} 
	{read "Read private objects"}
	{write "Edit all objects"}
	{delete "Delete all objects"}
	{km_publish "Publish an object if approval process is true"}
    }
}

ad_proc -public library_copy_data {source_package_id target_package_id} {
    Copy object type structure to the target package.
} {
    set ip_address [ns_conn peeraddr]
    set user_id [ad_get_user_id]

    db_exec_plsql copy_all_object_types {
	begin
	    library.copy (
		src_package_id => :source_package_id,
		dst_package_id => :target_package_id,
		creation_user => :user_id,
		creation_ip => :ip_address);
	end;
    }
}

ad_proc -public library_get_category_trees {package_id} {

    Returns a list of all category trees used by this instance.
    Used during community copying, to grant permissions on the trees.

} {

    return [db_list get_categories {
	-- Categories used for object type questions.
	select tree_id
	from sn_object_types ot, sn_question_object_type_map qm, sn_questions q
	where ot.context_id = :package_id
	    and qm.object_type_id = ot.object_type_id
	    and q.question_id = qm.question_id
	    and q.abstract_data_type in ('category', 'other_category')
	    and q.tree_id is not null
	union
	-- Categories used for object feedback.
	select object_id_two
	from acs_rels
	where rel_type = 'object_category_tree_rel'
	    and object_id_one = :package_id
	union
	-- Categories used on the PeopleShareNet application form.
	select object_id_two
	from acs_rels r, psn_category_trees ct
	where r.rel_type = 'object_category_tree_rel'
	    and ct.package_id = :package_id
	    and (r.object_id_one = ct.role_magic_id
		 or r.object_id_one = ct.language_magic_id
		 or r.object_id_one = ct.proficiency_magic_id)
	
    }]

}

namespace eval categories::packages::library {
    ad_proc -public get_instances { tree_id } {
	Returns a list of all library instances using a certain category tree
    } {
	set instance_list [list]
	set user_id [ad_conn user_id]

	db_foreach get_library_instances_for_tree {
	    select v.object_id, p.instance_name, v.object_name,
	           ci.community_id, p2.instance_name as community_name,
                   acs_permission.permission_p(p.package_id, :user_id, 'access') as show_p
	    from apm_packages p, apm_packages p2, sn_community_instances_all ci,
	         (select distinct q.question_id, ot.object_type_id as object_id,
	          ot.pretty_plural || ' (Question "' || q.pretty_name || '")' as object_name,
	          ot.context_id as package_id
	          from sn_object_types ot, sn_questions q,
	          sn_question_object_type_map qm, sw_category_dim d
	          where d.parent_node_id = q.node_id
	          and d.tree_id = :tree_id
	          and q.question_id = qm.question_id
	          and qm.object_type_id = ot.object_type_id
	          union
	          select 0 as question_id, p.package_id as object_id,
	                 'Feedback' as object_name, p.package_id
	          from acs_rels r, apm_packages p
	          where r.object_id_two = :tree_id
	          and r.rel_type = 'object_category_tree_rel'
	          and r.object_id_one = p.package_id
	          and p.package_key = 'library') v
	    where p.package_id = v.package_id
	    and ci.package_id = v.package_id
	    and p2.package_id = ci.community_id
	    order by p2.instance_name, p.instance_name, v.object_name
	} {
	    lappend instance_list [list Libraries $object_id $object_name $instance_name $community_id $community_name $show_p $show_p]
	}
	return $instance_list
    }
}

namespace eval sn_presentation::types::sn_object {
    ad_proc linkable_item {object_id url object_data_var} {
	upvar 1 $object_data_var object_data

	set public_p 0
	if { [string first "presentation=public" $object_data(presentation)] != -1 } {
	    set public_p 1
	}
	set archived_p 0
	if { [string first "archived" $object_data(presentation)] != -1 } {
	    set archived_p 1
	}
	set in_review_p 0
	if { [string first "in_review" $object_data(presentation)] != -1 } {
	    set in_review_p 1
	}

	set result "<a href=\"[ad_quotehtml $url]\">[sn_striphtml $object_data(name)]</a>"
	if { !$public_p } { append result " <font color=\"red\">\[Private\]</font>" }
	if { $archived_p } { append result " <font color=\"red\">\[Archived\]</font>" }
	if { $in_review_p } { append result " <font color=\"green\">\[In review\]</font>" }
	return $result
    }

    ad_proc item {object_id object_data_var} {
	upvar 1 $object_data_var object_data

	set public_p 0
	if { [string first "presentation=public" $object_data(presentation)] != -1 } {
	    set public_p 1
	}
	set archived_p 0
	if { [string first "archived" $object_data(presentation)] != -1 } {
	    set archived_p 1
	}
	set in_review_p 0
	if { [string first "in_review" $object_data(presentation)] != -1 } {
	    set in_review_p 1
	}

	set result [sn_striphtml $object_data(name)]
	if { !$public_p } { append result " <font color=\"red\">\[Private\]</font>" }
	if { $archived_p } { append result " <font color=\"red\">\[Archived\]</font>" }
	if { $in_review_p } { append result " <font color=\"green\">\[In review\]</font>" }
	return $result
    }
}

namespace eval alerts {
    namespace eval types {
	namespace eval sn_object {
	    ad_proc -public get_object {object_id} {
	    } {
		if { [catch { set object_as_clob [db_string object_public_object_clob \
			{select site_wide_search.indexed_content(:object_id) from dual}]} errmsg] } {
		    return [list "" 0]
		}
		return [list $object_as_clob 0]
	    }
	}
    }
}
