# /tcl/km-object-data.tcl
#
# $Id: km-object-data-procs.tcl,v 1.3 2009/02/13 20:40:39 jeffd Exp $

util_report_library_entry

# ********** Procs that get data from an object **********

ad_proc km_object_exists_p {object_id} "Returns 1 if this object exists in the library" {
    if [empty_string_p [km_get_object_type $object_id]] {
	return 0
    } else {
	return 1
    }
}

ad_proc km_get_object_type { object_id } "Returns the object type for any given object." {
    set this_object_type [db_string km_get_object_type "
	select object_type_id 
        from sn_objects 
        where object_id=:object_id" -default ""]

    return $this_object_type
}

ad_proc km_object_nephews_p { object_id } {
    Returns 1 if the object has a uncle-nephew question and some nephews.
} {
    set context_id [ad_conn package_id]
    set nephew_questions_without_nephews [db_string km_object_nephews_p {
	select count(*)
        from sn_questions q, sn_question_object_type_map qm, sn_objects o
        where q.question_id = qm.question_id
            and qm.question_state = 'active' 
	    and qm.mandatory_p = 't'
	    and qm.object_type_id = o.object_type_id
            and o.object_id = :object_id
            and o.context_id=:context_id
	    and q.abstract_data_type = 'nephew_object'
	    and not exists (select nephew.object_id
	                    from sn_objects nephew, sn_question_link_map map, sn_links l
	                    where l.object_id_a = :object_id
	                    and l.object_id_b = nephew.object_id
	                    and l.link_type = 'uncle_nephew'
	                    and l.link_id = map.link_id
	                    and map.question_id = q.question_id
	                    and nephew.expiration_date > sysdate)
    }]
    if {$nephew_questions_without_nephews > 0} {
	return 0
    } else {
	return 1
    }
}

ad_proc km_valid_nephew_publish_list_p { object_id nephew_list } {
    Returns 1 if for every uncle-nephew question of the object there is at least one
    nephew object in the list to be published.
} {
    set context_id [ad_conn package_id]
    set nephew_questions_without_nephews [db_string km_valid_nephew_publish_list_p "
	select count(*)
        from sn_questions q, sn_question_object_type_map qm, sn_objects o
        where q.question_id = qm.question_id
            and qm.question_state = 'active' 
	    and qm.mandatory_p = 't'
	    and qm.object_type_id = o.object_type_id
            and o.object_id = :object_id
            and o.context_id=:context_id
	    and q.abstract_data_type = 'nephew_object'
	    and not exists (select nephew.object_id
	                    from sn_objects nephew, sn_question_link_map map, sn_links l
	                    where l.object_id_a = :object_id
	                    and l.object_id_b = nephew.object_id
	                    and l.link_type = 'uncle_nephew'
	                    and l.link_id = map.link_id
	                    and map.question_id = q.question_id
	                    and nephew.expiration_date > sysdate
	                    and nephew.object_id in ([join $nephew_list ","]))
    "]
    if {$nephew_questions_without_nephews > 0} {
	return 0
    } else {
	foreach nephew_id $nephew_list {
	    if {![km_object_complete_p $nephew_id]} {
		return 0
	    }
	}
	return 1
    }
}

ad_proc km_uncle_nephew_questions { object_id } {
    Returns a list of all questions of the object (question_ids) that are of uncle-nephew type
} {
    set context_id [ad_conn package_id]
    set nephew_question_list [db_list_of_lists km_uncle_nephew_questions {
	select q.question_id, q.pretty_name
	from sn_objects o, sn_questions q, sn_question_object_type_map qm
        where q.question_id = qm.question_id
	    and qm.question_state = 'active'
	    and qm.mandatory_p = 't'
	    and qm.object_type_id = o.object_type_id
	    and o.object_id = :object_id
	    and o.context_id = :context_id
	    and q.abstract_data_type = 'nephew_object'
    }]
    return $nephew_question_list
}

ad_proc km_publishable_nephews { object_id question_id } {
    Returns the list of all publishable nephews for the object and question
} {
    set archived_p [db_string get_archived_status {select archived_p from sn_objects where object_id = :object_id}]
    set nephew_list [db_list_of_lists km_publishable_nephews {
	select nephew.object_id, nephew.one_line_description, nephew.in_review_p
	from sn_objects nephew, sn_links l, sn_question_link_map map
	where l.object_id_a = :object_id
	    and l.object_id_b = nephew.object_id
            and l.link_type = 'uncle_nephew'
            and l.link_id = map.link_id
            and map.question_id = :question_id
	    and nephew.archived_p = :archived_p
            and nephew.expiration_date > sysdate
    }]
    set result [list]
    foreach nephew $nephew_list {
	if {[km_object_complete_p [fst $nephew]]} {
	    lappend result $nephew
	}
    }
    return $result
}

ad_proc km_object_complete_p { object_id } {
    Returns 1 if the object is complete, 0 if not.
} {
    set required_questions [km_question_attributes [km_required_questions $object_id]]
    db_1row get_object_type_id {
	select object_type_id, decode(archived_p,'t',0,1) as unarchived_p
	from sn_objects
	where object_id = :object_id
    }
    if ![null_p $required_questions] {
	set questions {{question_id pretty_name abstract_data_type}}
	set questions [concat {{question_id pretty_name abstract_data_type}} $required_questions]
	set object_data [km_get_object_data -questions $questions -answers_only_p 0 -only_unarchived_p $unarchived_p $object_id]
    } else {
	return 1
    }
    foreach item $object_data {
	set value [fst $item]
	set abstract_data_type [lindex $item 3]
	set answered_p [km_answered_p $abstract_data_type $value]
	if !$answered_p {
	    set question_id [snd $item]
	    if {[km_active_path_p $question_id $object_id $object_type_id]} {
		return 0
	    }
	}
    }
    return 1
}

ad_proc km_get_object_summary { object_id } {
    Returns a tuple of object_id, public_p, archived_p, review_p, short_description, long_description of an object
} {
    set result [km_db_1row_to_list km_get_object_summary "
	select object_id, public_p, archived_p, in_review_p as review_p, one_line_description, overview 
        from sn_objects 
	where object_id=:object_id"]

    return $result
}

ad_proc km_get_object_name { object_id } {
    Returns the name or short_description of an object.
} {
    db_1row get_object_name {
	select one_line_description as object_name, object_type_id
	from sn_objects
	where object_id = :object_id
    }

    if [empty_string_p $object_name] {
	return "Unnamed [km_static object_type_pretty_name $object_type_id]"
    } else {
	return $object_name
    }
}

ad_proc km_get_object_data { 
    { 
	-answers_only_p 1
	-questions {}
        -show_values_p 1
	-only_unarchived_p 0
    }
    object_id 
} {

    Retrieves the data mapped to the given questions (as returned by 
    km_get_questions) and returns it as a list of lists in the form: 
    answer question_id pretty_question abstract_data_type branch_p
    or as just the answers if answers_only_p is set.

} {
    set user_id [ad_conn user_id]
    set archived_p [db_string get_archived_flag {select decode(archived_p,'t',1,0) from sn_objects where object_id = :object_id} -default 0]

    if [null_p $questions] {
	# Retrieve all possible questions for this object
	set question_states [list active deprecated read-only]
	if { $archived_p } {
	    lappend question_states invisible
	}
	set questions [km_get_questions -question_states $question_states -root_node_p 1 -branch_children_p 1 -all_properties_p 1 -object_id $object_id]

	# Get all text answers for all questions.
	# (So we don't have to query for each question.)
        if { $show_values_p} {
            set content_answers [km_get_object_content $object_id]
        } else {
            set content_answers ""
        }

	# Do the same for all linking questions.
	set linked_objects_from [km_get_linked_objects -from_p 1 -only_unarchived_p $only_unarchived_p $user_id $object_id]
	set linked_objects_to [km_get_linked_objects -to_p 1 -only_unarchived_p $only_unarchived_p $user_id $object_id]
	set link_question_id_ix [lsearch [head $linked_objects_from] "question_id"]
    } else {
	set question_ids [map fst [tail $questions]]
        if { $show_values_p} {
            set content_answers [km_get_object_content -question_ids $question_ids $object_id]
        } else {
            set content_answers ""
        }
	set linked_objects_to {}
	set linked_objects_from {}
	set link_question_id_ix -1
    }

    set key [fst $questions]
    set questions [tail $questions]

    set answer_list {}
    set question_id_ix [lsearch $key "question_id"]
    set pretty_name_ix [lsearch $key "pretty_name"]
    set abstract_data_type_ix [lsearch $key "abstract_data_type"]
    set question_state_ix [lsearch $key "question_state"]
    set branch_p_ix [lsearch $key "branch_p"]
    set root_branch_p_ix [lsearch $key "root_branch_p"]
    set references_question_id_ix [lsearch $key "references_question_id"]

    foreach question $questions {
	set question_id [lindex $question $question_id_ix]
	set pretty_name [lindex $question $pretty_name_ix]
	set abstract_data_type [lindex $question $abstract_data_type_ix]
        set references_question_id [lindex $question $references_question_id_ix]
	set question_state [lindex $question $question_state_ix]
	if { $branch_p_ix != -1 } {
	    set branch_p [lindex $question $branch_p_ix]
	} else {
	    set branch_p ""
	}
	if { $root_branch_p_ix != -1 } {
	    set root_branch_p [lindex $question $root_branch_p_ix]
	} else {
	    set root_branch_p ""
	}

	# Collect the data on the basis of the abstract_data_type
	switch $abstract_data_type {
	    "text" {
                if { $show_values_p} {
                    set text_value [lindex [snd [transpose $content_answers]] [lsearch [fst [transpose $content_answers]] $question_id]] 
                    set html_p [lindex [thd [transpose $content_answers]] [lsearch [fst [transpose $content_answers]] $question_id]] 
                    set answer [list $text_value $html_p]
                } else {
                    set text_value ""
                    set html_p ""
                    set answer ""
                }
	    }
            "integer" -
	    "date" -
	    "file"  { 
                if { $show_values_p} {
                    set answer [lindex [snd [transpose $content_answers]] [lsearch [fst [transpose $content_answers]] $question_id]]
                } else {
                    set answer ""
                }
	    }
	    "object_link" {
                if { $show_values_p} {
                    if { $link_question_id_ix == -1 } {
                        set answer [km_get_linked_objects -question_id $question_id -from_p 1 -only_unarchived_p $only_unarchived_p $user_id $object_id] 
                    } else {
			set answer [list [head $linked_objects_from]]
			foreach linked_object [tail $linked_objects_from] {
			    set link_question_id [lindex $linked_object $link_question_id_ix]
			    if { $link_question_id == $question_id } {
				lappend answer $linked_object
			    }
			}
			foreach linked_object [tail $linked_objects_to] {
			    set link_question_id [lindex $linked_object $link_question_id_ix]
			    if { $link_question_id == $references_question_id } {
				lappend answer $linked_object
			    }
			}
                    }
                } else {
                    set answer ""
                }
            }
	    "composite" {
		set child_ids [km_get_child_questions $question_id]
		if ![null_p $child_ids] {
		    set child_questions [km_get_questions -question_states {active deprecated read-only} -root_node_p 0 -question_ids $child_ids]
		    set answer [km_get_object_data -questions $child_questions -answers_only_p 0 -only_unarchived_p $only_unarchived_p $object_id]
		} else {
		    set answer {}
		}
	    }
	    "category" { set answer [km_get_categories $object_id $question_id] }
	    "other_category" { set answer [km_get_categories $object_id $question_id] }
	    "option" { set answer [km_get_object_answers $object_id $question_id] }
	    "user_link" { set answer [km8_get_to_links $object_id $question_id] }
	    "content_link" { set answer [km8_get_to_links -only_unarchived_p $only_unarchived_p $object_id $question_id] }
	    "child_object" {
		set answer ""
		set child_ids [km_get_child_objects -only_unarchived_p $only_unarchived_p $object_id $question_id]
		foreach child_id $child_ids {
		    lappend answer [km_get_object_summary $child_id]
		}
	    }
	    "nephew_object" {
		set answer ""
		set nephew_ids [km_get_nephew_objects -only_unarchived_p $only_unarchived_p $object_id $question_id]
		foreach nephew_id $nephew_ids {
		    lappend answer [km_get_object_summary $nephew_id]
		}
	    }
	    default { set answer "" }
	}
	if { $answers_only_p } { 
	    lappend answer_list $answer 
	} else {
	    lappend answer_list [list $answer $question_id $pretty_name $abstract_data_type $branch_p $root_branch_p]
	}
    }
    return $answer_list
}


ad_proc km_get_object_content { { -question_ids {} } object_id } {
    Returns all content about this object_id.
} {
    set result {}
    if ![null_p $question_ids] {
        foreach question_id $question_ids {
	    if {[db_0or1row km_get_object_content_10 "
		select content,decode(html_p,'t',1,0) as html_p from sn_content
            where object_id=:object_id and question_id=:question_id"]} {
                lappend result [list $question_id $content $html_p]
            }
        } 
    } else {
        set sql "select question_id, content,
                decode(html_p,'t',1,0) as html_p from sn_content
		where object_id=:object_id"
        db_foreach km_get_object_content_20 $sql {
            lappend result [list $question_id $content $html_p]
        } 
    }

    return $result
}

ad_proc km_get_object_answers { object_id question_id } {
    Returns the option_ids, pretty_names as tuples that answer this question for this object. 
} {
    set sql "select a.option_id, a.answer_option 
             from sn_answer_options a, sn_object_option_map m
             where a.option_id =m.option_id
             and m.object_id = :object_id
             and a.question_id = :question_id"
    set result [db_list_of_lists km_get_object_answers $sql]
    return $result
}

# ********** Saving data **********

ad_proc km_create_object { object_type_id user_id } {

    Creates a new knowledge object as a row in sn_objects. Note that the
    object is first considered completed when all required questions
    concerning the object have been answered.

} {
    set package_id [ad_conn package_id]
    set user_id [ad_conn user_id]
    set creation_ip [ad_conn peeraddr]
    
    set new_object_id [db_exec_plsql create_object_10 {
        begin
        :1 := object.insert_object (
        v_object_type_id => :object_type_id,
        v_creation_user => :user_id,
        v_creation_ip => :creation_ip,
        v_context_id => :package_id
        );
        end;
    }]

    return $new_object_id
}


ad_proc km_save_object_data { object_id user_id key_values } {
    Takes object data in as a list of keys value tuples and saves it in the database
    according to its abstract data type. 
} {
    if ![km_object_exists_p $object_id] { return }

    # Filter extra input field for category 'other'
    set other_category_key_values [km_filter_other_category_keys $key_values]
    set other_category_question_ids [fst [transpose $other_category_key_values]]
    set other_category_values [snd [transpose $other_category_key_values]]

    #Make sure every key is an integer because all tag names here are question_ids. 
    set key_values [km_filter_valid_keys $key_values]
    set submitted_question_ids [fst [transpose $key_values]]
    set values [snd [transpose $key_values]]
    if [null_p $submitted_question_ids] { return }

    db_1row km_save_object_data_10 "
	select long_description, public_until, start_date, end_date
	from sn_object_types
	where object_type_id = (select object_type_id from sn_objects 
				where object_id = :object_id)"

    set short_description [db_list_of_lists question_info_short_desc {
	select map.short_description
        from sn_types_map_short_name map
        where map.object_type_id = (select object_type_id from sn_objects 
				where object_id=:object_id)
        order by position
    }]

    #Get the attributes of the submitted_question_ids.
    set questions [km_question_attributes $submitted_question_ids]

    db_transaction {
        #Save the value according to the abstract_data_type
        foreach question $questions {
            
            set question_id [fst $question]
            set abstract_data_type [thd $question]
            set value_list [lindex $values [lsearch $submitted_question_ids $question_id]]

            switch $abstract_data_type {
                "category" { km_categorize_object $question_id $object_id $value_list }
                "other_category" {
		    km_categorize_object $question_id $object_id $value_list
		    if {[lsearch -exact $value_list [km_get_other_category_id $question_id]] == -1} {
			set other_value ""
		    } else {
			set other_value [fst [lindex $other_category_values [lsearch $other_category_question_ids $question_id]]]
		    }
		    km_save_object_content -html_p 0 $question_id $object_id $user_id $other_value
		}
                "option" { km_map_object_answers $question_id $object_id $value_list }
                "text" {
                    set content [fst $value_list]
                    set html_p [snd $value_list] 
                    km_save_object_content -html_p $html_p $question_id $object_id $user_id $content
                    km_update_descriptions \
                        -short_description $short_description -long_description $long_description \
                        -html_p $html_p $object_id $question_id $content
                }
                "file" {
                    set filename [fst $value_list]
                    if ![empty_string_p $filename] {
                        # First test if the file is empty. If so, we ignore it.

                        set tmpfilename [ns_queryget $question_id.tmpfile]
                        if [file size $tmpfilename] {
                            ns_cp $tmpfilename "[km_file_path]/$object_id.$question_id"

                            # switch from DOS (\) to UNIX (/) path separators.
                            regsub -all {\\} $filename "/" filename
                            set filename [file tail $filename]

                            km_save_object_content $question_id $object_id $user_id $filename
                            km_update_descriptions \
                                -short_description $short_description -long_description $long_description \
                                $object_id $question_id $filename
                        }
                    }
                }
                "integer" { 
                    km_save_object_content $question_id $object_id $user_id [fst $value_list] 
                    km_update_descriptions \
                        -short_description $short_description -long_description $long_description \
                        $object_id $question_id [fst $value_list]
                }
                "date" { 
                    if [date_p $value_list] {
                        km_save_object_content $question_id $object_id $user_id $value_list
                        if { $question_id == $public_until } {
                            db_dml km_save_object_data_20 "
			    update sn_objects 
			    set public_until=:value_list,
			        expired_warning_date = null
			    where object_id=:object_id"
                        }
                        if { $question_id == $start_date } {
                            db_dml km_save_object_data_22 "
			    update sn_objects 
			    set start_date=:value_list
			    where object_id=:object_id"
                        }
                        if { $question_id == $end_date } {
                            db_dml km_save_object_data_24 "
			    update sn_objects 
			    set end_date=:value_list
			    where object_id=:object_id"
                        }
                        km_update_descriptions \
                            -short_description $short_description -long_description $long_description \
                            $object_id $question_id [util_AnsiDatetoPrettyDate $value_list]
                    }
                }
                default { 
                    ns_log Error "Unknown abstract_data_type in km_save_object_data: $abstract_data_type" 
                }
            }
        }
    }
 
    return 
}

ad_proc km_append_object_content { {-html_p "f"} object_id question_id new_content } {

    Similar to km_save_object_content, but appends text to an already existing
    entry in sn_content.

} {
    set content [db_string km_append_object_content_1 "
	select content from sn_content
	where object_id=:object_id and question_id=:question_id" -default ""]

    append content $new_content

    km_save_object_content -html_p $html_p $question_id $object_id [ad_get_user_id] $content
}

ad_proc km_save_object_content { { -html_p ""}  question_id object_id user_id content } {
    Saves content into sn_content and and modification facts in sn_objects.
} {
    set user_id [ad_get_user_id]
    set ip_address [ns_conn peeraddr]
    db_transaction {
        if {[string length [string trim $content]] == 0} {
            db_dml km_save_object_content_2 {delete from sn_content
            where question_id=:question_id and object_id=:object_id
            }
            db_exec_plsql km_save_object_content_10 {
                begin
                object.audit_object (
                v_object_id              => :object_id,
                v_question_id	     => :question_id,
                v_last_modifying_user_id => :user_id,
                v_modifying_ip           => :ip_address,
                v_content                => :content
                );
                end;
            }
        } else {
            set html_p [ad_decode $html_p 0 "f" "t"]
            set content_length [string bytelength $content]
            
            db_exec_plsql km_save_object_content_10 {
                begin
                object.save_content (
                v_object_id           => :object_id,
                v_question_id	  => :question_id,
                v_html_p              => :html_p,
                v_content_length      => :content_length,
                v_modifying_user      => :user_id,
                v_modifying_ip        => :ip_address
                );
            end;
            }
            
            db_dml km_save_object_content_20 "
            update sn_content
            set content = empty_clob()
            where object_id = :object_id and question_id = :question_id
            returning content into :1"  -clobs [list $content]
            
            if {$content_length>4000} {
                set content [string range $content 0 2000]
            }
            
            db_exec_plsql km_save_object_content_10 {
                begin
                object.audit_object (
                v_object_id              => :object_id,
                v_question_id	     => :question_id,
                v_last_modifying_user_id => :user_id,
                v_modifying_ip           => :ip_address,
                v_content                => :content
                );
                end;
            }

        }
    }
}

ad_proc km_update_descriptions {
    {
	-short_description 0
	-long_description 0
	-html_p 0
    }
    object_id question_id content
} {
    Updates one_line_description and overview in sn_objects
    if the question_id matches the given IDs.
} {
    if { [lsearch $short_description $question_id] == -1 && $question_id != $long_description } {
	return
    }

    if [empty_string_p $html_p] { set html_p 0 }
    set content [util_trim_string_with_hrefs [string trim $content] 2000]
    
    if { [lsearch $short_description $question_id] != -1} {
        if {$short_description==$question_id} {
            set object_name [sn_striphtml $content]
            
            db_dml km_update_descriptions_10 {
                update sn_objects 
                set one_line_description = :object_name
                where object_id = :object_id
            }
        } else {
            set question1 [lindex $short_description 0]
            set question2 [lindex $short_description 1]
            set question3 [lindex $short_description 2]
            db_dml km_update_descriptions_15 {
                update sn_objects 
                set one_line_description = km_utilities.shortname_string(:object_id,:question1,:question2,:question3)
                where object_id=:object_id}
        }
    } elseif { $question_id == $long_description } {
        set decoded_html_p [ad_decode $html_p 0 "f" "t"]
	db_dml km_update_descriptions_20 {
	    update sn_objects
	    set overview = :content, overview_html_p = :decoded_html_p
	    where object_id = :object_id
	}
    }
}

ad_proc km_map_object_answers { question_id object_id option_ids } {
    Maps any number of answer_options to a
} {
     db_dml km_map_object_answers_1 "delete from sn_object_option_map where object_id=:object_id
               and option_id in ( select option_id from sn_answer_options 
                where question_id = :question_id)"
    foreach option_id $option_ids {
	set sql "insert into sn_object_option_map (object_id, option_id) values (:object_id, :option_id)"
	catch { [db_dml km_map_object_answers_2 $sql] }
    }	
}

ad_proc km_unoptionize_object { object_id question_id } {
   Removes options for this object and question
} {
    db_dml km_unoptionize_object_1 "
	delete from sn_object_option_map 
	where object_id = :object_id and
	    option_id in (select option_id from sn_answer_options where question_id = :question_id)"
    	
}

ad_proc km_delete_object_data { object_id question_id } {
    Removes the answer for a question. (Used for read-only questions.)
} {
    set abstract_data_type [db_string km_delete_object_data_1 "
	select abstract_data_type from sn_questions where question_id = :question_id"]

    switch $abstract_data_type {
	"text" -
	"integer" -
	"date" {
	    db_dml km_delete_object_data_2 "
		delete from sn_content where object_id=:object_id and question_id=:question_id"
	}
	"file" {
	    db_dml km_delete_object_data_3 "
		delete from sn_content where object_id=:object_id and question_id=:question_id"
	    ns_unlink -nocomplain "[km_file_path]/$object_id.$question_id"
	}
	"option" { km_unoptionize_object $object_id $question_id }
	"category" { km_uncategorize_object $object_id $question_id }
	"other_category" { km_uncategorize_object $object_id $question_id }
	default { ns_log Error "km_delete_object_data cannot handle abstract_data_type $abstract_data_type" }
    }
}


# ********** Filters/Validators **********

ad_proc km_filter_content_questions {question_ids abstract_data_types} {
    Returns a list of question_ids that are actually a row in sn_content.
} {
    set questions [zip $question_ids $abstract_data_types]
    set content [list]
    foreach pair $questions {
	set question_id [fst $pair]
	set abstract_data_type [snd $pair]
	if [lsearch [list text date upload] $abstract_data_type] >= 0 {
	    lappend filtered_questions $question_id
	}
    }
    return $filtered_questions
}

ad_proc integer_p {number} "returns 1 if this is an integer" {
    if [regexp {[^0-9]} $number] {
	return 0
    } else {
	return 1
    }
}

ad_proc filter_integers {char_list} {
    returns a filtered list of characters in the list that are integers
} {
    set result [list]
    foreach char $char_list {
	if [integer_p $char] {lappend result $char}
    }
    return $result
}

ad_proc date_p {date} "returns 1 if this is a date" {
    set result [catch {db_0or1row date_p "select to_date('$date') from dual"}]
    if {$result} {
	return 0
    } else {
	return 1
    }
}

ad_proc km_break_date {date} {
    Breaks an Oracle date in the form YYYY-MM-DD up into a list YYYY MM DD
} {
    if {![regexp {([0-9]*)-.*} $date match YYYY]} { return ""}
    if {![regexp {[0-9]*-([0-9]*)-*} $date match MM]} { return ""}
    if {![regexp {[0-9]*-[0-9]*-([0-9]*)} $date match DD]} {return ""}
    
    return [list $YYYY $MM $DD]
}

ad_proc empty_date_p {date} "returns 1 if this is an empty date in the form xx--" {

    if [regexp {.+-.+-.+} $date] {return 0}
    return 1
}

# Serve the abstract URL
# download/<object_id>.<question_id>/<client_filename>

proc km_serve_document {} {
    if { ![regexp {/download/([0-9]+)\.([0-9]+)/([^/]+)$} [ns_conn url] match object_id question_id client_filename] } {
        ad_return_warning "Malformed Document Request" \
            "Your request for a document was malformed."
        return
    }

    if [km_check_object_id $object_id] {
	ns_returnfile 200 [ns_guesstype $client_filename] "[km_file_path]/$object_id.$question_id"
    }
}

ad_proc set_the_usual_klib_context_bar { object_id {category_id ""} {pre_url ""} } {
    Display normal klib context bar - you have to use km_check_object_id first
} {
    set object_type_id [km_conn object_type_id]
    set object_name [km_conn object_name]
    set object_link "${pre_url}object-view?[export_url_vars object_id category_id]"
    set pretty_type_plural [km_static object_type_pretty_plural $object_type_id]
    if {[km_conn archived_p]} {
	set pretty_type_plural "$pretty_type_plural (Archived)"
	set archive t
    }
    set stripped_name [sn_striphtml $object_name]
    set browse_type "${pre_url}browse-one-type?[export_url_vars object_type_id archive]"
    set browse_category "${pre_url}browse-one-category?[export_url_vars object_type_id category_id archive]"
    set ancestors [reverse [km_get_object_ancestors $object_id]]

    if {![null_p $ancestors]} {
	set context_list [km_ancestors_context_list $ancestors $category_id $pre_url]
	lappend context_list [list $object_link $stripped_name]
	eval set_context_bar_data -link_last $context_list
    } elseif { ![empty_string_p $category_id] && ![string equal $category_id "none"] } {
	set_context_bar_data -link_last \
		[list $browse_type $pretty_type_plural] \
		[list $browse_category [km_category_name $category_id]] \
		[list $object_link $stripped_name]
    } else {
	set_context_bar_data -link_last [list $browse_type $pretty_type_plural] [list $object_link $stripped_name]
    }
}

ad_proc km_other_category_set { object_id question_id content } {
    Insert or update the text the user entered in the text field for category 'other'
} {
    if [empty_string_p $content] {
	km_other_category_delete $object_id $question_id
    } else {
	set content_length [string bytelength $content]
	set ip_address [ns_conn peeraddr]
	set user_id [ad_conn user_id]
	
	db_exec_plsql km_other_category_set_1 {
	    begin
	    object.save_content (
	    v_object_id           => :object_id,
	    v_question_id	  => :question_id,
	    v_html_p              => 'f',
	    v_content_length      => :content_length,
	    v_modifying_user      => :user_id,
	    v_modifying_ip        => :ip_address
	    );
	    end;
	}
	
	db_dml km_other_category_set_2 "
	update sn_content
	set content = empty_clob()
	where object_id = :object_id and question_id = :question_id
	returning content into :1"  -clobs [list $content]
	
	if {$content_length>4000} {
	    set content [string range $content 0 2000]
	}
	
	db_exec_plsql km_other_category_set_3 {
	    begin
	    object.audit_object (
	    v_object_id              => :object_id,
	    v_question_id	     => :question_id,
	    v_last_modifying_user_id => :user_id,
	    v_modifying_ip           => :ip_address,
	    v_content                => :content
	    );
	    end;
	}
    }
}

ad_proc km_other_category_get { object_id question_id } {
    Get the text the user entered in the text field for category 'other'
} {
    set result [db_string km_other_category_get {
	select content
	from sn_content
	where object_id = :object_id
	and question_id = :question_id
    } -default ""]

    return $result
}

ad_proc km_other_category_delete { object_id question_id } {
    Delete the text the user entered in the text field for category 'other'
} {
    set ip_address [ns_conn peeraddr]
    set user_id [ad_conn user_id]
    set content [km_other_category_get $object_id $question_id

    db_dml km_other_category_delete_1 {
	delete from sn_content
	where object_id = :object_id
	and question_id = :question_id
    }

    db_exec_plsql km_other_category_delete_2 {
	begin
	object.audit_object (
	v_object_id              => :object_id,
	v_question_id	         => :question_id,
	v_last_modifying_user_id => :user_id,
	v_modifying_ip           => :ip_address,
	v_content                => :content
	);
	end;
    }
}

ad_proc km_get_other_category_id { question_id } {
    get the category_id of the category "other" - or the last category in
    sorted list
} {
    set categories [db_list_of_lists check_other_category_id {
	select  /*+INDEX(sw_category_dim sw_category_dim_long_name) */ category.long_name(object_id)   as name,
	object_id as other_category_id
	from  sw_category_dim
	start with node_id = (select node_id from sn_questions where question_id = :question_id)
	connect by prior node_id = parent_node_id
    }]

    set values [fst [transpose $categories]]
    set ids [snd [transpose $categories]]
    set found_pos [lsearch $values "Other"]
    if {$found_pos > -1} {
	set other_category_id [lindex $ids $found_pos]
    } else {
	set other_category_id [snd [fst [reverse $categories]]]
    }
    return $other_category_id
}

util_report_successful_library_load 
